@@ -1,14 +1,34 @@
-from sqlalchemy import Integer, ForeignKey, String
-from sqlalchemy.types import PickleType, TypeDecorator, VARCHAR
-from sqlalchemy.orm import mapper, Session, composite
-from sqlalchemy.orm.mapper import Mapper
-from sqlalchemy.orm.instrumentation import ClassManager
-from sqlalchemy.testing.schema import Table, Column
-from sqlalchemy.testing import eq_, assert_raises_message, assert_raises
-from sqlalchemy.testing.util import picklers
-from sqlalchemy.testing import fixtures
+import copy
+import pickle
+
+from sqlalchemy import event
+from sqlalchemy import ForeignKey
+from sqlalchemy import func
+from sqlalchemy import Integer
+from sqlalchemy import String
+from sqlalchemy import util
 from sqlalchemy.ext.mutable import MutableComposite
-from sqlalchemy.ext.mutable import MutableDict, MutableList, MutableSet
+from sqlalchemy.ext.mutable import MutableDict
+from sqlalchemy.ext.mutable import MutableList
+from sqlalchemy.ext.mutable import MutableSet
+from sqlalchemy.orm import attributes
+from sqlalchemy.orm import column_property
+from sqlalchemy.orm import composite
+from sqlalchemy.orm import mapper
+from sqlalchemy.orm import Session
+from sqlalchemy.orm.instrumentation import ClassManager
+from sqlalchemy.orm.mapper import Mapper
+from sqlalchemy.testing import assert_raises
+from sqlalchemy.testing import assert_raises_message
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import mock
+from sqlalchemy.testing.schema import Column
+from sqlalchemy.testing.schema import Table
+from sqlalchemy.testing.util import picklers
+from sqlalchemy.types import PickleType
+from sqlalchemy.types import TypeDecorator
+from sqlalchemy.types import VARCHAR
 
 
 class Foo(fixtures.BasicEntity):
@@ -20,7 +40,6 @@ class SubFoo(Foo):
 
 
 class FooWithEq(object):
-
     def __init__(self, **kw):
         for k in kw:
             setattr(self, k, kw[k])
@@ -33,7 +52,6 @@ class FooWithEq(object):
 
 
 class Point(MutableComposite):
-
     def __init__(self, x, y):
         self.x = x
         self.y = y
@@ -52,13 +70,14 @@ class Point(MutableComposite):
         self.x, self.y = state
 
     def __eq__(self, other):
-        return isinstance(other, Point) and \
-            other.x == self.x and \
-            other.y == self.y
+        return (
+            isinstance(other, Point)
+            and other.x == self.x
+            and other.y == self.y
+        )
 
 
 class MyPoint(Point):
-
     @classmethod
     def coerce(cls, key, value):
         if isinstance(value, tuple):
@@ -79,7 +98,7 @@ class _MutableDictTestFixture(object):
 
 
 class _MutableDictTestBase(_MutableDictTestFixture):
-    run_define_tables = 'each'
+    run_define_tables = "each"
 
     def setup_mappers(cls):
         foo = cls.tables.foo
@@ -97,25 +116,42 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         assert_raises_message(
             ValueError,
             "Attribute 'data' does not accept objects of type",
-            Foo, data=set([1, 2, 3])
+            Foo,
+            data=set([1, 2, 3]),
         )
 
     def test_in_place_mutation(self):
         sess = Session()
 
-        f1 = Foo(data={'a': 'b'})
+        f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.commit()
 
-        f1.data['a'] = 'c'
+        f1.data["a"] = "c"
         sess.commit()
 
-        eq_(f1.data, {'a': 'c'})
+        eq_(f1.data, {"a": "c"})
+
+    def test_modified_event(self):
+        canary = mock.Mock()
+        event.listen(Foo.data, "modified", canary)
+
+        f1 = Foo(data={"a": "b"})
+        f1.data["a"] = "c"
+
+        eq_(
+            canary.mock_calls,
+            [
+                mock.call(
+                    f1, attributes.Event(Foo.data.impl, attributes.OP_MODIFIED)
+                )
+            ],
+        )
 
     def test_clear(self):
         sess = Session()
 
-        f1 = Foo(data={'a': 'b'})
+        f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.commit()
 
@@ -127,46 +163,46 @@ class _MutableDictTestBase(_MutableDictTestFixture):
     def test_update(self):
         sess = Session()
 
-        f1 = Foo(data={'a': 'b'})
+        f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.commit()
 
-        f1.data.update({'a': 'z'})
+        f1.data.update({"a": "z"})
         sess.commit()
 
-        eq_(f1.data, {'a': 'z'})
+        eq_(f1.data, {"a": "z"})
 
     def test_pop(self):
         sess = Session()
 
-        f1 = Foo(data={'a': 'b', 'c': 'd'})
+        f1 = Foo(data={"a": "b", "c": "d"})
         sess.add(f1)
         sess.commit()
 
-        eq_(f1.data.pop('a'), 'b')
+        eq_(f1.data.pop("a"), "b")
         sess.commit()
 
-        assert_raises(KeyError, f1.data.pop, 'g')
+        assert_raises(KeyError, f1.data.pop, "g")
 
-        eq_(f1.data, {'c': 'd'})
+        eq_(f1.data, {"c": "d"})
 
     def test_pop_default(self):
         sess = Session()
 
-        f1 = Foo(data={'a': 'b', 'c': 'd'})
+        f1 = Foo(data={"a": "b", "c": "d"})
         sess.add(f1)
         sess.commit()
 
-        eq_(f1.data.pop('a', 'q'), 'b')
-        eq_(f1.data.pop('a', 'q'), 'q')
+        eq_(f1.data.pop("a", "q"), "b")
+        eq_(f1.data.pop("a", "q"), "q")
         sess.commit()
 
-        eq_(f1.data, {'c': 'd'})
+        eq_(f1.data, {"c": "d"})
 
     def test_popitem(self):
         sess = Session()
 
-        orig = {'a': 'b', 'c': 'd'}
+        orig = {"a": "b", "c": "d"}
 
         # the orig dict remains unchanged when we assign,
         # but just making this future-proof
@@ -176,7 +212,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
         sess.commit()
 
         k, v = f1.data.popitem()
-        assert k in ('a', 'c')
+        assert k in ("a", "c")
         orig.pop(k)
 
         sess.commit()
@@ -186,45 +222,45 @@ class _MutableDictTestBase(_MutableDictTestFixture):
     def test_setdefault(self):
         sess = Session()
 
-        f1 = Foo(data={'a': 'b'})
+        f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.commit()
 
-        eq_(f1.data.setdefault('c', 'd'), 'd')
+        eq_(f1.data.setdefault("c", "d"), "d")
         sess.commit()
 
-        eq_(f1.data, {'a': 'b', 'c': 'd'})
+        eq_(f1.data, {"a": "b", "c": "d"})
 
-        eq_(f1.data.setdefault('c', 'q'), 'd')
+        eq_(f1.data.setdefault("c", "q"), "d")
         sess.commit()
 
-        eq_(f1.data, {'a': 'b', 'c': 'd'})
+        eq_(f1.data, {"a": "b", "c": "d"})
 
     def test_replace(self):
         sess = Session()
-        f1 = Foo(data={'a': 'b'})
+        f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.flush()
 
-        f1.data = {'b': 'c'}
+        f1.data = {"b": "c"}
         sess.commit()
-        eq_(f1.data, {'b': 'c'})
+        eq_(f1.data, {"b": "c"})
 
     def test_replace_itself_still_ok(self):
         sess = Session()
-        f1 = Foo(data={'a': 'b'})
+        f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.flush()
 
         f1.data = f1.data
-        f1.data['b'] = 'c'
+        f1.data["b"] = "c"
         sess.commit()
-        eq_(f1.data, {'a': 'b', 'b': 'c'})
+        eq_(f1.data, {"a": "b", "b": "c"})
 
     def test_pickle_parent(self):
         sess = Session()
 
-        f1 = Foo(data={'a': 'b'})
+        f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.commit()
         f1.data
@@ -234,7 +270,7 @@ class _MutableDictTestBase(_MutableDictTestFixture):
             sess = Session()
             f2 = loads(dumps(f1))
             sess.add(f2)
-            f2.data['a'] = 'c'
+            f2.data["a"] = "c"
             assert f2 in sess.dirty
 
     def test_unrelated_flush(self):
@@ -251,14 +287,24 @@ class _MutableDictTestBase(_MutableDictTestFixture):
     def _test_non_mutable(self):
         sess = Session()
 
-        f1 = Foo(non_mutable_data={'a': 'b'})
+        f1 = Foo(non_mutable_data={"a": "b"})
         sess.add(f1)
         sess.commit()
 
-        f1.non_mutable_data['a'] = 'c'
+        f1.non_mutable_data["a"] = "c"
         sess.commit()
 
-        eq_(f1.non_mutable_data, {'a': 'b'})
+        eq_(f1.non_mutable_data, {"a": "b"})
+
+    def test_copy(self):
+        f1 = Foo(data={"a": "b"})
+        f1.data = copy.copy(f1.data)
+        eq_(f1.data, {"a": "b"})
+
+    def test_deepcopy(self):
+        f1 = Foo(data={"a": "b"})
+        f1.data = copy.deepcopy(f1.data)
+        eq_(f1.data, {"a": "b"})
 
 
 class _MutableListTestFixture(object):
@@ -274,7 +320,7 @@ class _MutableListTestFixture(object):
 
 
 class _MutableListTestBase(_MutableListTestFixture):
-    run_define_tables = 'each'
+    run_define_tables = "each"
 
     def setup_mappers(cls):
         foo = cls.tables.foo
@@ -292,7 +338,8 @@ class _MutableListTestBase(_MutableListTestFixture):
         assert_raises_message(
             ValueError,
             "Attribute 'data' does not accept objects of type",
-            Foo, data=set([1, 2, 3])
+            Foo,
+            data=set([1, 2, 3]),
         )
 
     def test_in_place_mutation(self):
@@ -332,7 +379,7 @@ class _MutableListTestBase(_MutableListTestFixture):
         eq_(f1.data, [1, 4])
 
     def test_clear(self):
-        if not hasattr(list, 'clear'):
+        if not hasattr(list, "clear"):
             # py2 list doesn't have 'clear'
             return
         sess = Session()
@@ -385,6 +432,18 @@ class _MutableListTestBase(_MutableListTestFixture):
 
         eq_(f1.data, [1, 2, 5])
 
+    def test_operator_extend(self):
+        sess = Session()
+
+        f1 = Foo(data=[1, 2])
+        sess.add(f1)
+        sess.commit()
+
+        f1.data += [5]
+        sess.commit()
+
+        eq_(f1.data, [1, 2, 5])
+
     def test_insert(self):
         sess = Session()
 
@@ -460,6 +519,76 @@ class _MutableListTestBase(_MutableListTestFixture):
         sess.commit()
         eq_(f1.data[0], 3)
 
+    def test_copy(self):
+        f1 = Foo(data=[1, 2])
+        f1.data = copy.copy(f1.data)
+        eq_(f1.data, [1, 2])
+
+    def test_deepcopy(self):
+        f1 = Foo(data=[1, 2])
+        f1.data = copy.deepcopy(f1.data)
+        eq_(f1.data, [1, 2])
+
+    def test_legacy_pickle_loads(self):
+        # due to an inconsistency between pickle and copy, we have to change
+        # MutableList to implement a __reduce_ex__ method.   Which means we
+        # have to make sure all the old pickle formats are still
+        # deserializable since these can be used for persistence. these pickles
+        # were all generated using a MutableList that has only __getstate__ and
+        # __setstate__.
+
+        # f1 = Foo(data=[1, 2])
+        # pickles = [
+        #    dumps(f1.data)
+        #    for loads, dumps in picklers()
+        # ]
+        # print(repr(pickles))
+        # return
+
+        if util.py3k:
+            pickles = [
+                b"\x80\x04\x95<\x00\x00\x00\x00\x00\x00\x00\x8c\x16"
+                b"sqlalchemy.ext.mutable\x94\x8c\x0bMutableList\x94\x93\x94)"
+                b"\x81\x94(K\x01K\x02e]\x94(K\x01K\x02eb.",
+                b"ccopy_reg\n_reconstructor\np0\n(csqlalchemy.ext.mutable\n"
+                b"MutableList\np1\nc__builtin__\nlist\np2\n(lp3\nI1\naI2\n"
+                b"atp4\nRp5\n(lp6\nI1\naI2\nab.",
+                b"ccopy_reg\n_reconstructor\nq\x00(csqlalchemy.ext.mutable\n"
+                b"MutableList\nq\x01c__builtin__\nlist\nq\x02]q\x03(K\x01K"
+                b"\x02etq\x04Rq\x05]q\x06(K\x01K\x02eb.",
+                b"\x80\x02csqlalchemy.ext.mutable\nMutableList\nq\x00)\x81q"
+                b"\x01(K\x01K\x02e]q\x02(K\x01K\x02eb.",
+                b"\x80\x03csqlalchemy.ext.mutable\nMutableList\nq\x00)\x81q"
+                b"\x01(K\x01K\x02e]q\x02(K\x01K\x02eb.",
+                b"\x80\x04\x95<\x00\x00\x00\x00\x00\x00\x00\x8c\x16"
+                b"sqlalchemy.ext.mutable\x94\x8c\x0bMutableList\x94\x93\x94)"
+                b"\x81\x94(K\x01K\x02e]\x94(K\x01K\x02eb.",
+            ]
+        else:
+            pickles = [
+                "\x80\x02csqlalchemy.ext.mutable\nMutableList\nq\x00]q\x01"
+                "(K\x01K\x02e\x85q\x02Rq\x03.",
+                "\x80\x02csqlalchemy.ext.mutable\nMutableList"
+                "\nq\x00]q\x01(K\x01K\x02e\x85q\x02Rq\x03.",
+                "csqlalchemy.ext.mutable\nMutableList\np0\n"
+                "((lp1\nI1\naI2\natp2\nRp3\n.",
+                "csqlalchemy.ext.mutable\nMutableList\nq\x00(]"
+                "q\x01(K\x01K\x02etq\x02Rq\x03.",
+                "\x80\x02csqlalchemy.ext.mutable\nMutableList"
+                "\nq\x01]q\x02(K\x01K\x02e\x85Rq\x03.",
+                "\x80\x02csqlalchemy.ext.mutable\nMutableList\n"
+                "q\x01]q\x02(K\x01K\x02e\x85Rq\x03.",
+                "csqlalchemy.ext.mutable\nMutableList\np1\n"
+                "((lp2\nI1\naI2\natRp3\n.",
+                "csqlalchemy.ext.mutable\nMutableList\nq\x01"
+                "(]q\x02(K\x01K\x02etRq\x03.",
+            ]
+
+        for pickle_ in pickles:
+            obj = pickle.loads(pickle_)
+            eq_(obj, [1, 2])
+            assert isinstance(obj, MutableList)
+
 
 class _MutableSetTestFixture(object):
     @classmethod
@@ -474,7 +603,7 @@ class _MutableSetTestFixture(object):
 
 
 class _MutableSetTestBase(_MutableSetTestFixture):
-    run_define_tables = 'each'
+    run_define_tables = "each"
 
     def setup_mappers(cls):
         foo = cls.tables.foo
@@ -492,7 +621,8 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         assert_raises_message(
             ValueError,
             "Attribute 'data' does not accept objects of type",
-            Foo, data=[1, 2, 3]
+            Foo,
+            data=[1, 2, 3],
         )
 
     def test_clear(self):
@@ -545,6 +675,18 @@ class _MutableSetTestBase(_MutableSetTestFixture):
 
         eq_(f1.data, set([1, 2, 5]))
 
+    def test_binary_update(self):
+        sess = Session()
+
+        f1 = Foo(data=set([1, 2]))
+        sess.add(f1)
+        sess.commit()
+
+        f1.data |= set([2, 5])
+        sess.commit()
+
+        eq_(f1.data, set([1, 2, 5]))
+
     def test_intersection_update(self):
         sess = Session()
 
@@ -557,6 +699,18 @@ class _MutableSetTestBase(_MutableSetTestFixture):
 
         eq_(f1.data, set([2]))
 
+    def test_binary_intersection_update(self):
+        sess = Session()
+
+        f1 = Foo(data=set([1, 2]))
+        sess.add(f1)
+        sess.commit()
+
+        f1.data &= set([2, 5])
+        sess.commit()
+
+        eq_(f1.data, set([2]))
+
     def test_difference_update(self):
         sess = Session()
 
@@ -569,6 +723,18 @@ class _MutableSetTestBase(_MutableSetTestFixture):
 
         eq_(f1.data, set([1]))
 
+    def test_operator_difference_update(self):
+        sess = Session()
+
+        f1 = Foo(data=set([1, 2]))
+        sess.add(f1)
+        sess.commit()
+
+        f1.data -= set([2, 5])
+        sess.commit()
+
+        eq_(f1.data, set([1]))
+
     def test_symmetric_difference_update(self):
         sess = Session()
 
@@ -581,6 +747,18 @@ class _MutableSetTestBase(_MutableSetTestFixture):
 
         eq_(f1.data, set([1, 5]))
 
+    def test_binary_symmetric_difference_update(self):
+        sess = Session()
+
+        f1 = Foo(data=set([1, 2]))
+        sess.add(f1)
+        sess.commit()
+
+        f1.data ^= set([2, 5])
+        sess.commit()
+
+        eq_(f1.data, set([1, 5]))
+
     def test_remove(self):
         sess = Session()
 
@@ -637,6 +815,16 @@ class _MutableSetTestBase(_MutableSetTestFixture):
         sess.commit()
         eq_(f1.data, set([1, 2, 3]))
 
+    def test_copy(self):
+        f1 = Foo(data=set([1, 2]))
+        f1.data = copy.copy(f1.data)
+        eq_(f1.data, set([1, 2]))
+
+    def test_deepcopy(self):
+        f1 = Foo(data=set([1, 2]))
+        f1.data = copy.deepcopy(f1.data)
+        eq_(f1.data, set([1, 2]))
+
 
 class MutableColumnDefaultTest(_MutableDictTestFixture, fixtures.MappedTest):
     @classmethod
@@ -645,11 +833,12 @@ class MutableColumnDefaultTest(_MutableDictTestFixture, fixtures.MappedTest):
 
         mutable_pickle = MutableDict.as_mutable(PickleType)
         Table(
-            'foo', metadata,
+            "foo",
+            metadata,
             Column(
-                'id', Integer, primary_key=True,
-                test_needs_autoincrement=True),
-            Column('data', mutable_pickle, default={}),
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("data", mutable_pickle, default={}),
         )
 
     def setup_mappers(cls):
@@ -667,33 +856,33 @@ class MutableColumnDefaultTest(_MutableDictTestFixture, fixtures.MappedTest):
         sess.flush()
         assert isinstance(f1.data, self._type_fixture())
         assert f1 not in sess.dirty
-        f1.data['foo'] = 'bar'
+        f1.data["foo"] = "bar"
         assert f1 in sess.dirty
 
 
-
 class MutableWithScalarPickleTest(_MutableDictTestBase, fixtures.MappedTest):
-
     @classmethod
     def define_tables(cls, metadata):
         MutableDict = cls._type_fixture()
 
         mutable_pickle = MutableDict.as_mutable(PickleType)
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('skip', mutable_pickle),
-              Column('data', mutable_pickle),
-              Column('non_mutable_data', PickleType),
-              Column('unrelated_data', String(50))
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("skip", mutable_pickle),
+            Column("data", mutable_pickle),
+            Column("non_mutable_data", PickleType),
+            Column("unrelated_data", String(50)),
+        )
 
     def test_non_mutable(self):
         self._test_non_mutable()
 
 
 class MutableWithScalarJSONTest(_MutableDictTestBase, fixtures.MappedTest):
-
     @classmethod
     def define_tables(cls, metadata):
         import json
@@ -714,69 +903,149 @@ class MutableWithScalarJSONTest(_MutableDictTestBase, fixtures.MappedTest):
 
         MutableDict = cls._type_fixture()
 
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('data', MutableDict.as_mutable(JSONEncodedDict)),
-              Column('non_mutable_data', JSONEncodedDict),
-              Column('unrelated_data', String(50))
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("data", MutableDict.as_mutable(JSONEncodedDict)),
+            Column("non_mutable_data", JSONEncodedDict),
+            Column("unrelated_data", String(50)),
+        )
 
     def test_non_mutable(self):
         self._test_non_mutable()
 
 
-class MutableListWithScalarPickleTest(_MutableListTestBase, fixtures.MappedTest):
+class MutableColumnCopyJSONTest(_MutableDictTestBase, fixtures.MappedTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        import json
+        from sqlalchemy.ext.declarative import declarative_base
 
+        class JSONEncodedDict(TypeDecorator):
+            impl = VARCHAR(50)
+
+            def process_bind_param(self, value, dialect):
+                if value is not None:
+                    value = json.dumps(value)
+
+                return value
+
+            def process_result_value(self, value, dialect):
+                if value is not None:
+                    value = json.loads(value)
+                return value
+
+        MutableDict = cls._type_fixture()
+
+        Base = declarative_base(metadata=metadata)
+
+        class AbstractFoo(Base):
+            __abstract__ = True
+
+            id = Column(
+                Integer, primary_key=True, test_needs_autoincrement=True
+            )
+            data = Column(MutableDict.as_mutable(JSONEncodedDict))
+            non_mutable_data = Column(JSONEncodedDict)
+            unrelated_data = Column(String(50))
+
+        class Foo(AbstractFoo):
+            __tablename__ = "foo"
+            column_prop = column_property(
+                func.lower(AbstractFoo.unrelated_data)
+            )
+
+        assert Foo.data.property.columns[0].type is not AbstractFoo.data.type
+
+    def test_non_mutable(self):
+        self._test_non_mutable()
+
+
+class MutableColumnCopyArrayTest(_MutableListTestBase, fixtures.MappedTest):
+    __requires__ = ("array_type",)
+
+    @classmethod
+    def define_tables(cls, metadata):
+        from sqlalchemy.ext.declarative import declarative_base
+        from sqlalchemy.sql.sqltypes import ARRAY
+
+        MutableList = cls._type_fixture()
+
+        Base = declarative_base(metadata=metadata)
+
+        class Mixin(object):
+            data = Column(MutableList.as_mutable(ARRAY(Integer)))
+
+        class Foo(Mixin, Base):
+            __tablename__ = "foo"
+            id = Column(Integer, primary_key=True)
+
+
+class MutableListWithScalarPickleTest(
+    _MutableListTestBase, fixtures.MappedTest
+):
     @classmethod
     def define_tables(cls, metadata):
         MutableList = cls._type_fixture()
 
         mutable_pickle = MutableList.as_mutable(PickleType)
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('skip', mutable_pickle),
-              Column('data', mutable_pickle),
-              Column('non_mutable_data', PickleType),
-              Column('unrelated_data', String(50))
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("skip", mutable_pickle),
+            Column("data", mutable_pickle),
+            Column("non_mutable_data", PickleType),
+            Column("unrelated_data", String(50)),
+        )
 
 
 class MutableSetWithScalarPickleTest(_MutableSetTestBase, fixtures.MappedTest):
-
     @classmethod
     def define_tables(cls, metadata):
         MutableSet = cls._type_fixture()
 
         mutable_pickle = MutableSet.as_mutable(PickleType)
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('skip', mutable_pickle),
-              Column('data', mutable_pickle),
-              Column('non_mutable_data', PickleType),
-              Column('unrelated_data', String(50))
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("skip", mutable_pickle),
+            Column("data", mutable_pickle),
+            Column("non_mutable_data", PickleType),
+            Column("unrelated_data", String(50)),
+        )
 
 
-class MutableAssocWithAttrInheritTest(_MutableDictTestBase,
-                                      fixtures.MappedTest):
-
+class MutableAssocWithAttrInheritTest(
+    _MutableDictTestBase, fixtures.MappedTest
+):
     @classmethod
     def define_tables(cls, metadata):
 
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('data', PickleType),
-              Column('non_mutable_data', PickleType),
-              Column('unrelated_data', String(50))
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("data", PickleType),
+            Column("non_mutable_data", PickleType),
+            Column("unrelated_data", String(50)),
+        )
 
-        Table('subfoo', metadata,
-              Column('id', Integer, ForeignKey('foo.id'), primary_key=True),
-              )
+        Table(
+            "subfoo",
+            metadata,
+            Column("id", Integer, ForeignKey("foo.id"), primary_key=True),
+        )
 
     def setup_mappers(cls):
         foo = cls.tables.foo
@@ -789,46 +1058,49 @@ class MutableAssocWithAttrInheritTest(_MutableDictTestBase,
     def test_in_place_mutation(self):
         sess = Session()
 
-        f1 = SubFoo(data={'a': 'b'})
+        f1 = SubFoo(data={"a": "b"})
         sess.add(f1)
         sess.commit()
 
-        f1.data['a'] = 'c'
+        f1.data["a"] = "c"
         sess.commit()
 
-        eq_(f1.data, {'a': 'c'})
+        eq_(f1.data, {"a": "c"})
 
     def test_replace(self):
         sess = Session()
-        f1 = SubFoo(data={'a': 'b'})
+        f1 = SubFoo(data={"a": "b"})
         sess.add(f1)
         sess.flush()
 
-        f1.data = {'b': 'c'}
+        f1.data = {"b": "c"}
         sess.commit()
-        eq_(f1.data, {'b': 'c'})
+        eq_(f1.data, {"b": "c"})
 
 
-class MutableAssociationScalarPickleTest(_MutableDictTestBase,
-                                         fixtures.MappedTest):
-
+class MutableAssociationScalarPickleTest(
+    _MutableDictTestBase, fixtures.MappedTest
+):
     @classmethod
     def define_tables(cls, metadata):
         MutableDict = cls._type_fixture()
         MutableDict.associate_with(PickleType)
 
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('skip', PickleType),
-              Column('data', PickleType),
-              Column('unrelated_data', String(50))
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("skip", PickleType),
+            Column("data", PickleType),
+            Column("unrelated_data", String(50)),
+        )
 
 
-class MutableAssociationScalarJSONTest(_MutableDictTestBase,
-                                       fixtures.MappedTest):
-
+class MutableAssociationScalarJSONTest(
+    _MutableDictTestBase, fixtures.MappedTest
+):
     @classmethod
     def define_tables(cls, metadata):
         import json
@@ -850,27 +1122,33 @@ class MutableAssociationScalarJSONTest(_MutableDictTestBase,
         MutableDict = cls._type_fixture()
         MutableDict.associate_with(JSONEncodedDict)
 
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('data', JSONEncodedDict),
-              Column('unrelated_data', String(50))
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("data", JSONEncodedDict),
+            Column("unrelated_data", String(50)),
+        )
 
 
-class CustomMutableAssociationScalarJSONTest(_MutableDictTestBase,
-                                             fixtures.MappedTest):
+class CustomMutableAssociationScalarJSONTest(
+    _MutableDictTestBase, fixtures.MappedTest
+):
 
     CustomMutableDict = None
 
     @classmethod
     def _type_fixture(cls):
-        if not(getattr(cls, 'CustomMutableDict')):
+        if not (getattr(cls, "CustomMutableDict")):
             MutableDict = super(
-                CustomMutableAssociationScalarJSONTest, cls)._type_fixture()
+                CustomMutableAssociationScalarJSONTest, cls
+            )._type_fixture()
 
             class CustomMutableDict(MutableDict):
                 pass
+
             cls.CustomMutableDict = CustomMutableDict
         return cls.CustomMutableDict
 
@@ -895,12 +1173,15 @@ class CustomMutableAssociationScalarJSONTest(_MutableDictTestBase,
         CustomMutableDict = cls._type_fixture()
         CustomMutableDict.associate_with(JSONEncodedDict)
 
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('data', JSONEncodedDict),
-              Column('unrelated_data', String(50))
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("data", JSONEncodedDict),
+            Column("unrelated_data", String(50)),
+        )
 
     def test_pickle_parent(self):
         # Picklers don't know how to pickle CustomMutableDict,
@@ -909,26 +1190,29 @@ class CustomMutableAssociationScalarJSONTest(_MutableDictTestBase,
 
     def test_coerce(self):
         sess = Session()
-        f1 = Foo(data={'a': 'b'})
+        f1 = Foo(data={"a": "b"})
         sess.add(f1)
         sess.flush()
         eq_(type(f1.data), self._type_fixture())
 
 
 class _CompositeTestBase(object):
-
     @classmethod
     def define_tables(cls, metadata):
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('x', Integer),
-              Column('y', Integer),
-              Column('unrelated_data', String(50))
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("x", Integer),
+            Column("y", Integer),
+            Column("unrelated_data", String(50)),
+        )
 
     def setup(self):
         from sqlalchemy.ext import mutable
+
         mutable._setup_composite_listener()
         super(_CompositeTestBase, self).setup()
 
@@ -944,17 +1228,20 @@ class _CompositeTestBase(object):
         return Point
 
 
-class MutableCompositeColumnDefaultTest(_CompositeTestBase,
-                                        fixtures.MappedTest):
+class MutableCompositeColumnDefaultTest(
+    _CompositeTestBase, fixtures.MappedTest
+):
     @classmethod
     def define_tables(cls, metadata):
         Table(
-            'foo', metadata,
-            Column('id', Integer, primary_key=True,
-                   test_needs_autoincrement=True),
-            Column('x', Integer, default=5),
-            Column('y', Integer, default=9),
-            Column('unrelated_data', String(50))
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("x", Integer, default=5),
+            Column("y", Integer, default=9),
+            Column("unrelated_data", String(50)),
         )
 
     @classmethod
@@ -963,9 +1250,11 @@ class MutableCompositeColumnDefaultTest(_CompositeTestBase,
 
         cls.Point = cls._type_fixture()
 
-        mapper(Foo, foo, properties={
-            'data': composite(cls.Point, foo.c.x, foo.c.y)
-        })
+        mapper(
+            Foo,
+            foo,
+            properties={"data": composite(cls.Point, foo.c.x, foo.c.y)},
+        )
 
     def test_evt_on_flush_refresh(self):
         # this still worked prior to #3427 being fixed in any case
@@ -982,16 +1271,17 @@ class MutableCompositeColumnDefaultTest(_CompositeTestBase,
 
 
 class MutableCompositesUnpickleTest(_CompositeTestBase, fixtures.MappedTest):
-
     @classmethod
     def setup_mappers(cls):
         foo = cls.tables.foo
 
         cls.Point = cls._type_fixture()
 
-        mapper(FooWithEq, foo, properties={
-            'data': composite(cls.Point, foo.c.x, foo.c.y)
-        })
+        mapper(
+            FooWithEq,
+            foo,
+            properties={"data": composite(cls.Point, foo.c.x, foo.c.y)},
+        )
 
     def test_unpickle_modified_eq(self):
         u1 = FooWithEq(data=self.Point(3, 5))
@@ -1000,16 +1290,15 @@ class MutableCompositesUnpickleTest(_CompositeTestBase, fixtures.MappedTest):
 
 
 class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
-
     @classmethod
     def setup_mappers(cls):
         foo = cls.tables.foo
 
         Point = cls._type_fixture()
 
-        mapper(Foo, foo, properties={
-            'data': composite(Point, foo.c.x, foo.c.y)
-        })
+        mapper(
+            Foo, foo, properties={"data": composite(Point, foo.c.x, foo.c.y)}
+        )
 
     def test_in_place_mutation(self):
         sess = Session()
@@ -1031,7 +1320,7 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         sess.commit()
 
         f1.data
-        assert 'data' in f1.__dict__
+        assert "data" in f1.__dict__
         sess.close()
 
         for loads, dumps in picklers():
@@ -1057,7 +1346,10 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         assert_raises_message(
             ValueError,
             "Attribute 'data' does not accept objects",
-            setattr, f1, 'data', 'foo'
+            setattr,
+            f1,
+            "data",
+            "foo",
         )
 
     def test_unrelated_flush(self):
@@ -1074,7 +1366,6 @@ class MutableCompositesTest(_CompositeTestBase, fixtures.MappedTest):
 
 
 class MutableCompositeCallableTest(_CompositeTestBase, fixtures.MappedTest):
-
     @classmethod
     def setup_mappers(cls):
         foo = cls.tables.foo
@@ -1083,9 +1374,13 @@ class MutableCompositeCallableTest(_CompositeTestBase, fixtures.MappedTest):
 
         # in this case, this is not actually a MutableComposite.
         # so we don't expect it to track changes
-        mapper(Foo, foo, properties={
-            'data': composite(lambda x, y: Point(x, y), foo.c.x, foo.c.y)
-        })
+        mapper(
+            Foo,
+            foo,
+            properties={
+                "data": composite(lambda x, y: Point(x, y), foo.c.x, foo.c.y)
+            },
+        )
 
     def test_basic(self):
         sess = Session()
@@ -1099,9 +1394,9 @@ class MutableCompositeCallableTest(_CompositeTestBase, fixtures.MappedTest):
         eq_(f1.data.x, 3)
 
 
-class MutableCompositeCustomCoerceTest(_CompositeTestBase,
-                                       fixtures.MappedTest):
-
+class MutableCompositeCustomCoerceTest(
+    _CompositeTestBase, fixtures.MappedTest
+):
     @classmethod
     def _type_fixture(cls):
 
@@ -1113,9 +1408,9 @@ class MutableCompositeCustomCoerceTest(_CompositeTestBase,
 
         Point = cls._type_fixture()
 
-        mapper(Foo, foo, properties={
-            'data': composite(Point, foo.c.x, foo.c.y)
-        })
+        mapper(
+            Foo, foo, properties={"data": composite(Point, foo.c.x, foo.c.y)}
+        )
 
     def test_custom_coerce(self):
         f = Foo()
@@ -1134,18 +1429,22 @@ class MutableCompositeCustomCoerceTest(_CompositeTestBase,
 
 
 class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
-
     @classmethod
     def define_tables(cls, metadata):
-        Table('foo', metadata,
-              Column('id', Integer, primary_key=True,
-                     test_needs_autoincrement=True),
-              Column('x', Integer),
-              Column('y', Integer)
-              )
-        Table('subfoo', metadata,
-              Column('id', Integer, ForeignKey('foo.id'), primary_key=True),
-              )
+        Table(
+            "foo",
+            metadata,
+            Column(
+                "id", Integer, primary_key=True, test_needs_autoincrement=True
+            ),
+            Column("x", Integer),
+            Column("y", Integer),
+        )
+        Table(
+            "subfoo",
+            metadata,
+            Column("id", Integer, ForeignKey("foo.id"), primary_key=True),
+        )
 
     @classmethod
     def setup_mappers(cls):
@@ -1154,9 +1453,9 @@ class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
 
         Point = cls._type_fixture()
 
-        mapper(Foo, foo, properties={
-            'data': composite(Point, foo.c.x, foo.c.y)
-        })
+        mapper(
+            Foo, foo, properties={"data": composite(Point, foo.c.x, foo.c.y)}
+        )
         mapper(SubFoo, subfoo, inherits=Foo)
 
     def test_in_place_mutation_subclass(self):
@@ -1179,7 +1478,7 @@ class MutableInheritedCompositesTest(_CompositeTestBase, fixtures.MappedTest):
         sess.commit()
 
         f1.data
-        assert 'data' in f1.__dict__
+        assert "data" in f1.__dict__
         sess.close()
 
         for loads, dumps in picklers():
